@@ -156,11 +156,28 @@ sub authorize {
 		'proxy-authenticate' => 'proxy-authorization',
 		'www-authenticate' => 'authorization',
 	);
+
 	while ( my ($req,$resp) = each %auth_map ) {
+		my $existing_auth;
 		if ( my @auth = $response->get_header_hashval( $req ) ) {
 			foreach my $a (@auth) {
 				my $h = $a->{parameter};
 
+				# check if we already have an authorize header for this realm/opaque
+				if ( ! $existing_auth ) {
+					$existing_auth = {};
+					foreach my $hdr ( $self->get_header_hashval( $resp )) {
+						my @auth = grep { defined } map { $hdr->{parameter}{$_} }qw( realm opaque );
+						$existing_auth->{ join( "\0",@auth ) } = 1;
+					}
+				}
+
+				my @auth = grep { defined } map { $h->{$_} }qw( realm opaque );
+				if ( $existing_auth->{ join( "\0",@auth ) } ) {
+					# we have this auth header already, don't repeat
+					next;
+				}
+
 				# RFC2617
 				# we support only md5 (not md5-sess or other)
 				# and only empty qop or qop=auth (not auth-int or other)
@@ -190,10 +207,10 @@ sub authorize {
 					uri => $self->uri,
 				);
 				$digest{opaque} = $h->{opaque} if defined $h->{opaque};
-
+						
 				# 3.2.2.1
 				if ( $h->{qop} ) {
-					my $nc = $digest{nc} = 1;
+					my $nc = $digest{nc} = '00000001';
 					my $cnonce = $digest{cnonce} = sprintf("%08x",rand(2**32));
 					$digest{qop} = $h->{qop};
 					$digest{response} = md5_hex( join(':',
@@ -215,11 +232,12 @@ sub authorize {
 
 				# RFC2617 has it's specific ideas what should be quoted and what not
 				# so we assemble it manually
-				my $header = qq[Digest username="$digest{username}", realm="$digest{realm}",].
-					qq[ nonce="$digest{nonce}", uri=$digest{uri}, response="$digest{response}"];
-				$header.= qq[, opaque="$digest{opaque}"] if defined $digest{opaque};
-				$header.= qq[, cnonce="$digest{cnonce}"] if defined $digest{cnonce};
-				$header.= qq[, qop=$digest{qop}] if defined $digest{qop};
+				my $header = qq[Digest username="$digest{username}",realm="$digest{realm}",].
+					qq[nonce="$digest{nonce}",uri="$digest{uri}",response="$digest{response}"];
+				$header.= qq[,opaque="$digest{opaque}"] if defined $digest{opaque};
+				$header.= qq[,cnonce="$digest{cnonce}"] if defined $digest{cnonce};
+				$header.= qq[,qop=$digest{qop}] if defined $digest{qop};
+				$header.= qq[,nc=$digest{nc}] if defined $digest{nc};
 				$self->add_header( $resp, $header );
 				$auth++;
 			}
